<?php

session_start();


function validateEmail($email) {
	if(filter_var($email, FILTER_VALIDATE_EMAIL)) {
		return true;
	} else {
		return false;
	}
}

function validatePassword($password) {
	if(strlen($password) > 4) {
		return true;
	} else {
		return false;
	}
}

/*** Hashování ***/
function myHash($string) {
	return password_hash($string, PASSWORD_BCRYPT);
}

/*** Fce pro odesílání emailu. ***/
function myMail($email, $subject, $message) {
	return mail($email, $subject, $message, "Content-type: text/plain; charset=utf-8\r\nFROM: php_machr@napisecku.cz");
}


class User {
	/*** Načtení dat z DB ***/
	public static function getUser($email) {
		static $data;

		if(isset($data)) {
			 return $data;
		}
		$data = DatabaseManager::getUser($email) or null;

		return $data;
	}

	/*** Ověření při pokusu přihlásit se ***/
	public static function verifyUser($email, $password) {
		$data = self::getUser($email);

		if($data && $data['verify']) {
			return password_verify($password, $data['passHash']);
		}
		return false;
	}

	/*** Registrace uživatele ***/
	public static function setRegistration($email, $password) {
		if(!self::getUser($email)) {
			$passHash = myHash($password);
			$hash = myHash(microtime());
			DatabaseManager::setRegistration($email, $passHash, $hash);
			myMail($email, "Registrace PHP_machr..", "Potvrďte registraci emailu " . $email . " pomocí odkazu http://php_machr.napisecku.cz/verify.php?" . $hash);
			return true;
		} else {
			return false;
		}
	}

	/*** Potvrzení registrace ***/
	public static function verifyRegistration($hash) {
		$data = DatabaseManager::getRegistration($hash);
		if($data){
			DatabaseManager::verifyRegistration($data['id']);
			return $data['email'];
		} else {
			return  false;
		}
	}

	/*** Změna hesla uživatele ***/
	public  static function changePassword($hash, $password) {
		$data = DatabaseManager::getLostRegistration($hash);

		if($data) {
			DatabaseManager::setPassword($data['email'], myHash($password));
			return true;
		} else {
			return false;
		}
	}

	/*** Zapomenuté heslo ***/
	public static function lostPassword($email) {
		$data = self::getUser($email);

		if($data) {
			$hash = myHash(microtime());
			DatabaseManager::lostPassword($email, $hash);
			myMail($email, "Obnovahesla PHP_machr..", "Odkaz pro obnovuzapomenutého hesla: http://php_machr.napisecku.cz/heslo.php?" . $hash);
			return true;
		} else {
			return false;
		}
	}
}

/*** Třída pro práci s DB ***/
class DatabaseManager {

	private static function getConnection() {
		static $hDB;

		if(isset($hDB)) {
			return $hDB;
		}

		$hDB = new PDO("mysql:host=localhost;dbname=php_machr;charset=utf8","php_machr","machr");
		
		return $hDB;
	}

	/*** Zavedení uživatele do DB (bez ověření). ***/
	public static function setRegistration($email, $passHash, $hash) {
		$prepare = self::getConnection()->prepare("INSERT INTO registrace (email, passHash, hash) VALUES (:email, :passHash, :hash)");
		$prepare->execute(array(':email' => $email, ':passHash' =>  $passHash, ':hash' => $hash));
	}

	/*** Ověřování existence registračního hashe v DB. ***/
	public static function getRegistration($hash) {
		$prepare = self::getConnection()->prepare("SELECT * FROM registrace WHERE hash = :hash");
		$prepare->execute(array(':hash' => $hash));

		return $prepare->fetch(PDO::FETCH_ASSOC);
	}

	/*** Vyhledání klíče při obnově hesla ***/
	public static function getLostRegistration($hash) {
		$prepare = self::getConnection()->prepare("SELECT * FROM registrace WHERE restore = :hash");
		$prepare->execute(array(':hash' => $hash));

		return $prepare->fetch(PDO::FETCH_ASSOC);
	}

	/*** Přepsis ověření(udělí ověření zadanému uživateli). ***/
	public static function verifyRegistration($id) {
		$prepare = self::getConnection()->prepare("UPDATE registrace SET verify = 1, hash = null WHERE id = :id");
		$prepare->execute(array(':id' => $id));
	}

	/*** Smazání uživatele z DB. ***/
	public static function  removeRegistration($id) {
		$prepare = self::getConnection()->prepare("DELETE FROM registrace WHERE id = :id");
		$prepare->execute(array(':id' => $id));
	}

	/*** Načtení dat o uživateli (pro ověření hesla při loginu). ***/
	public static function getUser($email) {
		$prepare = self::getConnection()->prepare("SELECT * FROM registrace WHERE email = :email");
		$prepare->execute(array(':email' =>  $email));

		return $prepare->fetch(PDO::FETCH_ASSOC);
	}

	/*** Změna hesla v DB. ***/
	public static function setPassword($email, $passHash) {
		$prepare = self::getConnection()->prepare("UPDATE registrace SET passHash = :passHash, restore  =  null  WHERE email = :email");
		$prepare->execute(array(':passHash' => $passHash, ':email' => $email));
	}

	/*** Změna hesla ***/
	public static function lostPassword($email, $hash) {
		$prepare = self::getConnection()->prepare("UPDATE registrace SET restore  =  :hash  WHERE email = :email");
		$prepare->execute(array(':hash' => $hash, ':email' => $email));
	}
}












?>